@@ -7,6 +7,7 @@
 #include <glib.h>
 #include <gtk/gtk.h>
 #include <libintl.h>
+#include <stdlib.h>
 
 #define HOME_DIR g_get_home_dir()
 
@@ -14,6 +15,9 @@
 
 #define DESKTOP_CMD_EXEC_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE (obj, DESKTOP_CMD_EXEC_TYPE, DesktopCmdExecPrivate))
 
+#define NON_GTK_RESPONSE_ADD_CMD 10
+#define NON_GTK_RESPONSE_DEL_CMD 20
+
 void desktop_cmd_exec_write_settings (DesktopCmdExec *self);
 gboolean desktop_cmd_exec_update_content (DesktopCmdExec *self);
 
@@ -29,13 +33,15 @@ struct _DesktopCmdExecPrivate
 	
 	gboolean isPressed;
 
-	gchar * c1_title;
-	gchar * c1_command;
+	/* new */
+	gchar ** c_titles;
+	gchar ** c_commands;
+	gint c_size;
+	gint c_current;
 	
 	gboolean c1_updOnClick;
 	gboolean c1_updOnDesktop;
 	
-	
 	guint eventID;
 };
 
@@ -56,20 +62,46 @@ void desktop_cmd_exec_read_settings ( DesktopCmdExec *self )
 	if (fileExists) {
 		GError *error=NULL;
 
-		self->priv->c1_command = g_key_file_get_string (keyFile, "config", "c1_command", &error);
+		//BEGIN get list
+		self->priv->c_commands = (gchar **) g_key_file_get_string_list (keyFile, "config", "c_commands", &(self->priv->c_size) ,&error);
 		if (error) {
-			self->priv->c1_command = NULL;
+			self->priv->c_commands = NULL;
+			self->priv->c_size = 0;
 			g_error_free (error);
 			error = NULL;
 		}
 
-		self->priv->c1_title = g_key_file_get_string (keyFile, "config", "c1_title", &error);
+		gint consistencyCheck = -1;
+		self->priv->c_titles = g_key_file_get_string_list (keyFile, "config", "c_titles", &consistencyCheck ,&error);
 		if (error) {
-			self->priv->c1_title = NULL;
+			self->priv->c_titles = NULL;
+			consistencyCheck = 0;
 			g_error_free (error);
 			error = NULL;
 		}
 
+		self->priv->c_current = g_key_file_get_integer (keyFile, "config", "c_current", &error);
+		if (error) {
+			self->priv->c_current = 0;
+			g_error_free (error);
+			error = NULL;
+		}
+
+		if(consistencyCheck != self->priv->c_size)
+		{
+			g_warning("Settings file corrupted!");
+			self->priv->c_commands = NULL;
+			self->priv->c_titles = NULL;
+			self->priv->c_size = 0;
+			self->priv->c_current = 0;
+		}
+		else if (consistencyCheck == 0)
+		{
+			g_warning("No settings file, creating one");
+		}
+		//END
+
+
 		self->priv->c1_updOnClick = g_key_file_get_boolean (keyFile, "config", "c1_updOnClick", &error);
 		if (error) {
 			self->priv->c1_updOnClick = TRUE;
@@ -85,11 +117,13 @@ void desktop_cmd_exec_read_settings ( DesktopCmdExec *self )
 		}
 
 	} else {
-		
-		self->priv->c1_command = NULL;
-		self->priv->c1_title = NULL;
 		self->priv->c1_updOnClick = TRUE;
 		self->priv->c1_updOnDesktop = TRUE;
+
+		self->priv->c_commands = NULL;
+		self->priv->c_titles = NULL;
+		self->priv->c_size = 0;
+		self->priv->c_current = 0;
 		
 		desktop_cmd_exec_write_settings(self);
 	}
@@ -108,14 +142,30 @@ void desktop_cmd_exec_write_settings (DesktopCmdExec *self)
 	gchar *filename;
 	
 	keyFile = g_key_file_new();
-	
-	if(!self->priv->c1_command)
-		self->priv->c1_command = "cat /proc/uptime | awk '{print $1}'";
-	g_key_file_set_string (keyFile, "config", "c1_command", self->priv->c1_command);
 
-	if(!self->priv->c1_title)
-		self->priv->c1_title = "Uptime: ";
-	g_key_file_set_string (keyFile, "config", "c1_title", self->priv->c1_title);
+	if( (!self->priv->c_commands) || self->priv->c_size == 0)
+	{
+// 		g_warning("records empty, filling with pre-loaded vals");
+		//records empty, filling with pre-loaded vals
+		self->priv->c_size = 4;
+		gchar * p_titles[] = { "Uptime", "Battery", "Uptime(sb)", "home size(sb)"};
+		gchar * p_commands[] = { "uptime | awk 'NF == 7 {print $2 \" \" $3}; NF == 11 {print $3 \" \" $4 \" \" $5}'",
+		"lshal | grep battery.reporting | awk -F. '{print $3}' | awk '$1 == \"current\" { curval = $3}; $1 == \"design\" {print curval \"/\" $3}'",
+		"cat /proc/uptime | awk '{print $1}'",
+		"ls -lha | awk '$1 == \"total\" {print $2}'"};
+		self->priv->c_titles = p_titles;
+		self->priv->c_commands = p_commands;
+		self->priv->c_current = 2;
+
+	}
+	
+	g_key_file_set_string_list(keyFile, "config", "c_titles", (const gchar **)(self->priv->c_titles),self->priv->c_size);
+	g_key_file_set_string_list(keyFile, "config", "c_commands", (const gchar **)(self->priv->c_commands),self->priv->c_size);
+	g_key_file_set_integer(keyFile, "config", "c_current", self->priv->c_current);
+	
+// 	g_warning("Size %d, Curr %d", self->priv->c_size, self->priv->c_current);
+
+// 	g_warning("Saved::! %s", self->priv->c_titles[self->priv->c_current]);
 
 	g_key_file_set_boolean (keyFile, "config", "c1_updOnClick", self->priv->c1_updOnClick);
 	g_key_file_set_boolean (keyFile, "config", "c1_updOnDesktop", self->priv->c1_updOnDesktop);
@@ -128,6 +178,7 @@ void desktop_cmd_exec_write_settings (DesktopCmdExec *self)
 	g_key_file_free (keyFile);
 	g_free (fileData);
 	g_free (filename);
+	desktop_cmd_exec_read_settings (self);
 }
 
 void desktop_cmd_exec_button_press (GtkWidget *widget, GdkEventButton *event, DesktopCmdExec *self)
@@ -202,48 +253,52 @@ gboolean desktop_cmd_exec_update_content (DesktopCmdExec *self)
 	gchar *result;
 	
 	gboolean found = FALSE;
+	if(self->priv->c_current >= self->priv->c_size)
+	{
+		gtk_label_set_text (GTK_LABEL (self->priv->cmdTitle_lb), "Error:");
+		gtk_label_set_text (GTK_LABEL (self->priv->cmdResult_lb), "No commands");
+		return 0;
+	}
+// 	g_warning("To exec %s", self->priv->c_titles[self->priv->c_current]);
 
-	gtk_label_set_text (GTK_LABEL (self->priv->cmdTitle_lb), self->priv->c1_title);
+	gtk_label_set_text (GTK_LABEL (self->priv->cmdTitle_lb), self->priv->c_titles[self->priv->c_current]);
 
-	fp = popen (self->priv->c1_command, "r");
+	fp = popen (self->priv->c_commands[self->priv->c_current], "r");
 	
 	while (fgets (line, sizeof line, fp)) {
 		result = g_strstrip (line);
 		gtk_label_set_text (GTK_LABEL (self->priv->cmdResult_lb), result);
+		found = TRUE;
 	}
-	found = TRUE;
 	pclose (fp);
 	
 	if (!found) {
-		gtk_label_set_text (GTK_LABEL (self->priv->cmdResult_lb), "result probs");
+		gtk_label_set_text (GTK_LABEL (self->priv->cmdResult_lb), "Invalid Command");
 	}
 	
 	return found;
 }
 
-void desktop_cmd_exec_settings (HDHomePluginItem *hitem, DesktopCmdExec *self)
+void desktop_cmd_exec_add_cmd ( DesktopCmdExec *self)
 {
-// 	g_warning ("desktop_cmd_exec_settings");
-	GtkWidget *dialog = gtk_dialog_new_with_buttons ("Desktop Command Execution Widget Settings", NULL, 0, _("wdgt_bd_save"), GTK_RESPONSE_ACCEPT, NULL);
-	
+	GtkWidget *dialog = gtk_dialog_new_with_buttons ("Add custom command", NULL, 0,
+																									 _("wdgt_bd_save"), GTK_RESPONSE_ACCEPT, NULL);
 	GtkWidget *content_area = gtk_dialog_get_content_area (GTK_DIALOG (dialog));
 	GtkSizeGroup *group = GTK_SIZE_GROUP (gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL));
-
+	
 	GtkWidget *titleBox = gtk_hbox_new (FALSE, 0);
 	GtkWidget *titleLabel = gtk_label_new ("Title:");
 	gtk_misc_set_alignment (GTK_MISC (titleLabel), 0, 0.5);
 	gtk_misc_set_padding (GTK_MISC (titleLabel), HILDON_MARGIN_DOUBLE, 0);
 	gtk_size_group_add_widget (group, titleLabel);
-	
+
 	GtkWidget *titleEntry = hildon_entry_new (HILDON_SIZE_AUTO_WIDTH | HILDON_SIZE_FINGER_HEIGHT);
-	hildon_entry_set_text (HILDON_ENTRY (titleEntry), self->priv->c1_title);
 	gtk_box_pack_start (GTK_BOX (titleBox), titleLabel, FALSE, FALSE, 0);
 	gtk_box_pack_start (GTK_BOX (titleBox), titleEntry, TRUE, TRUE, 0);
 
 	gtk_container_add (GTK_CONTAINER (content_area), titleBox);
 
 
-	
 	GtkWidget *commandBox = gtk_hbox_new (FALSE, 0);
 	GtkWidget *commandLabel = gtk_label_new ("Command:");
 	gtk_misc_set_alignment (GTK_MISC (commandLabel), 0, 0.5);
@@ -251,48 +306,138 @@ void desktop_cmd_exec_settings (HDHomePluginItem *hitem, DesktopCmdExec *self)
 	gtk_size_group_add_widget (group, commandLabel);
 
 	GtkWidget *commandEntry = hildon_entry_new (HILDON_SIZE_AUTO_WIDTH | HILDON_SIZE_FINGER_HEIGHT);
-	hildon_entry_set_text (HILDON_ENTRY (commandEntry), self->priv->c1_command);
 	gtk_box_pack_start (GTK_BOX (commandBox), commandLabel, FALSE, FALSE, 0);
 	gtk_box_pack_start (GTK_BOX (commandBox), commandEntry, TRUE, TRUE, 0);
-	
+
 	gtk_container_add (GTK_CONTAINER (content_area), commandBox);
-
-	GtkHBox *hbox_upds = GTK_HBOX (gtk_hbox_new (FALSE, 0));
-	
-	GtkWidget *toggleOnClick = hildon_gtk_toggle_button_new (HILDON_SIZE_AUTO_WIDTH | HILDON_SIZE_FINGER_HEIGHT);
-	gtk_button_set_label (GTK_BUTTON (toggleOnClick), "@Click");
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (toggleOnClick), self->priv->c1_updOnClick);
-	gtk_container_add (GTK_CONTAINER (hbox_upds), toggleOnClick);
-
-	GtkWidget *toggleOnDesktop = hildon_gtk_toggle_button_new (HILDON_SIZE_AUTO_WIDTH | HILDON_SIZE_FINGER_HEIGHT);
-	gtk_button_set_label (GTK_BUTTON (toggleOnDesktop), "@Desktop");
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (toggleOnDesktop), self->priv->c1_updOnDesktop);
-	gtk_container_add (GTK_CONTAINER (hbox_upds), toggleOnDesktop);
-
-	GtkWidget *updBox = gtk_hbox_new (FALSE, 0);
-	GtkWidget *updLabel = gtk_label_new ("Updade:");
-	gtk_misc_set_alignment (GTK_MISC (updLabel), 0, 0.5);
-	gtk_misc_set_padding (GTK_MISC (updLabel), HILDON_MARGIN_DOUBLE, 0);
-	gtk_size_group_add_widget (group, updLabel);
-	gtk_box_pack_start (GTK_BOX (updBox), updLabel, FALSE, FALSE, 0);
-	gtk_box_pack_start (GTK_BOX (updBox), GTK_WIDGET (hbox_upds), TRUE, TRUE, 0);
-	
-	gtk_container_add (GTK_CONTAINER (content_area), updBox);
-
 	gtk_widget_show_all (dialog);
-	
+
+
 	if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT) {
-		self->priv->c1_title = g_strdup (hildon_entry_get_text (HILDON_ENTRY (titleEntry)));
-		self->priv->c1_command = g_strdup (hildon_entry_get_text (HILDON_ENTRY (commandEntry)));
-		self->priv->c1_updOnClick = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (toggleOnClick));
-		self->priv->c1_updOnDesktop = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (toggleOnDesktop));
+
+// 		g_warning("Adding: %s", g_strdup (hildon_entry_get_text (HILDON_ENTRY (titleEntry))));
+// 		g_warning("cmd: %s", g_strdup (hildon_entry_get_text (HILDON_ENTRY (commandEntry))));
+		
+		gchar ** newTitles = malloc( sizeof(gchar*)*(self->priv->c_size+1) );
+		gchar ** newCommands = malloc( sizeof(gchar*)*(self->priv->c_size+1) );
+		
+		int i;
+		for(i=0;i<self->priv->c_size;i++)
+		{
+			newTitles[i] = g_strdup(self->priv->c_titles[i]);
+			newCommands[i] = g_strdup(self->priv->c_commands[i]);
+		}
+		
+		newTitles[self->priv->c_size] = g_strdup (hildon_entry_get_text (HILDON_ENTRY (titleEntry)));
+		newCommands[self->priv->c_size] = g_strdup (hildon_entry_get_text (HILDON_ENTRY (commandEntry)));
+
+		self->priv->c_current = self->priv->c_size;
+		self->priv->c_size += 1;
+
+
+
+		self->priv->c_titles = newTitles;
+		self->priv->c_commands = newCommands;
 
 		desktop_cmd_exec_write_settings (self);
+		
 		desktop_cmd_exec_update_content (self);
 	}
+	
 	gtk_widget_destroy (dialog);
 }
 
+void desktop_cmd_exec_settings (HDHomePluginItem *hitem, DesktopCmdExec *self)
+{
+// 	g_warning ("desktop_cmd_exec_settings");
+
+	int settingsRunning = 1;
+	while(settingsRunning)
+	{
+		GtkWidget *dialog = gtk_dialog_new_with_buttons ("Desktop Command Execution Widget Settings", NULL, 0,
+																										_("wdgt_bd_save"), GTK_RESPONSE_ACCEPT,
+																										"Add Entry", NON_GTK_RESPONSE_ADD_CMD,
+																										//"Delete Entry", NON_GTK_RESPONSE_DEL_CMD,
+																										 NULL);
+
+		GtkWidget *content_area = gtk_dialog_get_content_area (GTK_DIALOG (dialog));
+		GtkSizeGroup *group = GTK_SIZE_GROUP (gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL));
+
+		GtkWidget *selector = hildon_touch_selector_new_text ();
+		int i;
+		for(i=0;i<self->priv->c_size;i++)
+		{
+			hildon_touch_selector_append_text (HILDON_TOUCH_SELECTOR (selector), self->priv->c_titles[i]);
+		}
+		hildon_touch_selector_set_active (HILDON_TOUCH_SELECTOR (selector), 0, self->priv->c_current);
+
+
+		GtkWidget * cmdSelector = hildon_picker_button_new (HILDON_SIZE_AUTO_WIDTH | HILDON_SIZE_FINGER_HEIGHT, HILDON_BUTTON_ARRANGEMENT_HORIZONTAL);
+		hildon_button_set_title (HILDON_BUTTON (cmdSelector), "Preloaded Commands");
+		hildon_button_add_title_size_group (HILDON_BUTTON (cmdSelector), group);
+		hildon_button_set_alignment (HILDON_BUTTON (cmdSelector), 0, 0.5, 0, 0);
+		hildon_picker_button_set_selector (HILDON_PICKER_BUTTON (cmdSelector), HILDON_TOUCH_SELECTOR (selector));
+		gtk_container_add (GTK_CONTAINER (content_area), cmdSelector);
+
+		GtkHBox *hbox_upds = GTK_HBOX (gtk_hbox_new (FALSE, 0));
+
+		GtkWidget *toggleOnClick = hildon_gtk_toggle_button_new (HILDON_SIZE_AUTO_WIDTH | HILDON_SIZE_FINGER_HEIGHT);
+		gtk_button_set_label (GTK_BUTTON (toggleOnClick), "@Click");
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (toggleOnClick), self->priv->c1_updOnClick);
+		gtk_container_add (GTK_CONTAINER (hbox_upds), toggleOnClick);
+
+		GtkWidget *toggleOnDesktop = hildon_gtk_toggle_button_new (HILDON_SIZE_AUTO_WIDTH | HILDON_SIZE_FINGER_HEIGHT);
+		gtk_button_set_label (GTK_BUTTON (toggleOnDesktop), "@Desktop");
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (toggleOnDesktop), self->priv->c1_updOnDesktop);
+		gtk_container_add (GTK_CONTAINER (hbox_upds), toggleOnDesktop);
+
+		GtkWidget *updBox = gtk_hbox_new (FALSE, 0);
+		GtkWidget *updLabel = gtk_label_new ("Updade:");
+		gtk_misc_set_alignment (GTK_MISC (updLabel), 0, 0.5);
+		gtk_misc_set_padding (GTK_MISC (updLabel), HILDON_MARGIN_DOUBLE, 0);
+		gtk_size_group_add_widget (group, updLabel);
+		gtk_box_pack_start (GTK_BOX (updBox), updLabel, FALSE, FALSE, 0);
+		gtk_box_pack_start (GTK_BOX (updBox), GTK_WIDGET (hbox_upds), TRUE, TRUE, 0);
+
+		gtk_container_add (GTK_CONTAINER (content_area), updBox);
+
+		gtk_widget_show_all (dialog);
+
+		int setDialogReturn = 0;
+
+		setDialogReturn = gtk_dialog_run (GTK_DIALOG (dialog));
+		if (setDialogReturn == GTK_RESPONSE_ACCEPT)
+		{
+			int cmdSel = hildon_touch_selector_get_active (HILDON_TOUCH_SELECTOR (selector), 0);
+// 			g_warning("Selected: %d", cmdSel);
+
+			if(cmdSel < self->priv->c_size && cmdSel >= 0)
+			{
+				self->priv->c_current = (gint) cmdSel;
+			}
+			self->priv->c1_updOnClick = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (toggleOnClick));
+			self->priv->c1_updOnDesktop = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (toggleOnDesktop));
+
+			desktop_cmd_exec_write_settings (self);
+			desktop_cmd_exec_update_content (self);
+			settingsRunning = 0;
+		}
+		else if(setDialogReturn == NON_GTK_RESPONSE_ADD_CMD)
+		{
+			desktop_cmd_exec_add_cmd(self);
+			
+			for(i=0;i<self->priv->c_size;i++)
+			{
+				hildon_touch_selector_append_text (HILDON_TOUCH_SELECTOR (selector), self->priv->c_titles[i]);
+			}
+			hildon_touch_selector_set_active (HILDON_TOUCH_SELECTOR (selector), 0, self->priv->c_current);
+		}
+		else
+			settingsRunning = 0;
+	gtk_widget_destroy (dialog);
+	}
+}
+
 static void desktop_cmd_exec_check_desktop (GObject *gobject, GParamSpec *pspec, DesktopCmdExec *self)
 {
 	if (self->priv->c1_updOnClick) {
@@ -317,7 +462,9 @@ static void desktop_cmd_exec_finalize (GObject *object)
 	if (self->priv->eventID) {
 		g_source_remove (self->priv->eventID);
 	}
-// 	g_object_unref (self->priv->connection);
+
+	g_strfreev(self->priv->c_titles);
+	g_strfreev(self->priv->c_commands);
 
 	G_OBJECT_CLASS (desktop_cmd_exec_parent_class)->finalize (object);
 }
